from Device import *


class SC01(Device):
    allophones = (
        'EH3', 'EH2', 'EH1', 'PA0', 'DT', 'A2', 'A1', 'ZH', 'AH2', 'I3', 'I2',
        'I1', 'M', 'N', 'B', 'V', 'CH', 'SH', 'Z', 'AW1', 'NG', 'AH1', 'OO1',
        'OO', 'L', 'K', 'J', 'H', 'G', 'F', 'D', 'S', 'A', 'AY', 'Y1', 'UH3',
        'AH', 'P', 'O', 'I', 'U', 'Y', 'T', 'R', 'E', 'W', 'AE', 'AE1', 'AW2',
        'UH2', 'UH1', 'UH', 'O2', 'O1', 'IU', 'U1', 'THV', 'TH', 'ER', 'EH',
        'E1', 'AW', 'PA1', 'STOP')

    def __init__(self, system, addr_range, addr_bus_width):
        Device.__init__(self, addr_range, addr_bus_width)
        self.system = system

    def reset(self):
        self.output = []
        self.write_tick = 0

    def read(self, addr):
        raise NotImplemented("Read from SC01")

    def write(self, addr, val):
        if self.trace:
            print("SC01: {} ({:02x})".format(self.all_str(val), val))
        self.output.append(val)
        self.write_tick = self.system.ticks

    def tick(self, val):
        if self.write_tick:
            if val > self.write_tick + 8000:
                self.system.cpu.irq_pending = True
                self.write_tick = 0

    def __repr__(self):
        ret = [self.all_str(a) for a in self.output]
        return "Output: "+str(ret)

    @classmethod
    def all_str(cls, a):
            if a < len(cls.allophones):
                return cls.allophones[a]
            else:
                return '???'
