#!/bin/env python3

from System import *
from ACIA import *
from CPU import *
from RAM import *
from ROM import *
from SC01 import *


def enableTrace(system):
    system.cpu.trace = True
    system.dump(0x100)


def dumpWordBuf(system):
    system.dump(0, 1)


def printWordBuf(system):
    val = ""
    addr = 2
    while True:
        b = system.read(addr)
        if b >= 0x7F:
            break
        val += chr(b)
        addr += 1
    print(val, file=system.outh)
    print(val)


def dumpPtr(system):
    system.dump(0x26, 1)


def beforeSub3(system):
    rulePtr = system.read_be_word(0x26)
    print("before sub3: rulePtr: {:04x}".format(rulePtr))
    return rulePtr == 0x7988


def specialRule(system):
    return system.cpu.reg.acc_a == 0x08


def recvSp(system):
    return system.cpu.reg.acc_a == 0x20


def skipNext(system):
    print("before skipNext: rulePtr: {:04x}".format(system.read_be_word(0x26)))


def emptyQueue(system):
    system.devices['SC01'].trace = True
    return True


def dumpAllo(system):
    allo = []
    addr = system.cpu.reg.ind_x
    b = system.read(addr)
    while (b & 0x80) == 0:
        allo.append(SC01.allophones[b & 0x3F])
        addr += 1
        b = system.read(addr)
    print(allo, file=system.outh)
    print(allo)
    return False


def exampleFile(system, fname):
    fileh = open(fname, encoding='latin-1')
    system.devices['ACIA'].readFrom(fileh)

    system.outh = open('allophones-'+fname, 'w', buffering=1)

    system.cpu.addBreak(0x6cbc, dumpAllo) # dump allophones
    system.cpu.addBreak(0x7cbc, dumpAllo) # dump allophones

    system.cpu.addBreak(0x6fe0) # break when queue empty
    system.cpu.addBreak(0x7fe0) # break when queue empty

    #system.cpu.addBreak(0x6fe0, emptyQueue) # break when queue empty
    #system.cpu.addBreak(0x7fe0, emptyQueue) # break when queue empty

    #system.cpu.addBreak(0x6c4b, printWordBuf) # dump before tx2gr
    #system.cpu.addBreak(0x7c4b, printWordBuf) # dump before tx2gr

    #system.cpu.addBreak(0x6a7a) # line to proc
    #system.cpu.addBreak(0x7a7a) # line to proc

    #system.cpu.addBreak(0x6b86, enableTrace) #
    #system.cpu.addBreak(0x7b86, enableTrace) #

    system.run()


def test(system):
    #system.devices['ACIA'].addInput("one two three\r")

    system.cpu.addBreak(0x6fe0, emptyQueue) # break when queue empty
    system.cpu.addBreak(0x7fe0, emptyQueue) # break when queue empty

    #system.cpu.addBreak(0x6b73, enableTrace)
    system.cpu.addBreak(0x7b73, enableTrace)
    #system.cpu.addBreak(0x6d23, beforeSub3)
    #system.cpu.addBreak(0x7d23, beforeSub3)
    #system.cpu.addBreak(0x6d93, skipNext)
    #system.cpu.addBreak(0x7d93, skipNext)

    system.run()

    system.devices['ACIA'].addInput("7\r")

    #system.cpu.addBreak(0x6c70, enableTrace)
    #system.cpu.addBreak(0x7c70, enableTrace)
    system.cpu.addBreak(0x6dfe, specialRule)
    system.cpu.addBreak(0x7dfe, specialRule)

    system.run()


if __name__ == '__main__':
    system = System(15)
    system.add('6802', M6802(system))
    system.add('RAM', RAM((0, 0x1FFF), 10))
    system.add('ACIA', ACIA(system, (0x2000, 0x3FFF), 1))
    system.add('SC01', SC01(system, (0x4000, 0x5FFF), 0))
    system.add('ROM', ROM('typetalk.bin', (0x6000, 0x7FFF), 12))

    system.reset()
    #system.devices['SC01'].trace = True

    #exampleFile(system, 'example.txt')
    #exampleFile(system, 'dickens.txt')
    #exampleFile(system, '../rewrite/test.txt')
    test(system)

    #system.addWriteWatch(0x90, {'val': 0x12, 'halt': True})
    #system.addWriteWatch(0x4a)
    #system.cpu.addBreak(0x6c70, enableTrace)
    #system.cpu.addBreak(0x7c70, enableTrace)
    #system.cpu.addBreak(0x6d2c)
    #system.cpu.addBreak(0x7d2c)

    #system.run(50000)
    #if not system.halt:
        #system.devices['ACIA'].addInput("\x1B\x15") # Caps mode on
        #system.devices['ACIA'].addInput("FFA\r")
    #if not system.halt:
        #system.run(110000)
